<?php
/**
* Подпрограмма для получения сведений о домене
*
* @package      seoUtilitiesFiles
* @author       Andrej Panov <ab@cdef.ru>
* @copyright    2009 Andrej Panov
* @license      http://www.opensource.org/licenses/lgpl-license.php LGPL
* @version      1.20111118
* @link         http://a-panov.ru/
* @link         http://a-panov.ru/download/
* @since        2009-09-22
*/
 
/**
* Класс для формирования запроса на получение сведений о домене
*
* @package  seoUtilitiesClasses
*/
class cWhois {
static public
    /**
    * @var int $errnum номер ошибки последнего запроса
    */
    $errnum = 0;
    
    /**
    * Получение информации по запрашиваемому домену
    *
    * @param array $murl массив, возвращённый функцией cBase::get()
    * @return string|bool whois-информация либо false при ошиблке
    */
    static function get($murl) {
        self::$errnum = 0;
        
        // если вместо хоста задан IP, даже не пытаемся определить whois
        if (isset($murl['ip'])) return self::_error(1);
        
        // если имя домена задано с ошибкой (например, museum), возвращаем ложь
        if ( ! isset($murl['root_domain'])) return self::_error(2);
        
        // узнаём сервер, предоставляющий whois-сервис для заданного домена
        $buf = self::_gurl('whois.iana.org', 43, $murl['root_domain']);
        if (false === $buf) return self::_error(3);
        
        /*
        preg_match('!Whois Server \(port ([0-9]+)\):[^a-zA-Z0-9\.\-]*([a-zA-Z0-9\.\-]+)!', $buf, $t);
        if (empty($t[2])) return self::_error(4);
        $port   = (int) $t[1];
        $server = trim($t[2]);
        */
        # в районе июля 2010 изменился алгоритм выдачи
        preg_match('!whois:[\\x00-\\x20]+([a-zA-Z0-9\.\-]+)!', $buf, $t);
        $port = 43;
        $server = trim($t[1]);
        
        // если задан домен 2 уровня, просто получаем данные по нему
        if ($murl['domain_level'] == 2) {
            $buf = self::_gurl($server, $port, $murl['host']);
            if (false === $buf) return self::_error(5);
            
            if ($buf) $response = self::_gresponse($buf) ? $buf : false;
            else $response = false;
        } else {
            // если домен 2 или 3 уровня, получаем данные в 2 захода:
            // 1. пытаемся получить информацию по домену 3 уровня.
            // 2. получаем информацию по домену 2 уровня.
            // сделано для того, чтобы не вводить анализ таких зон, как:
            // ru.com, net.ru, pp.ru и т. п.
            
            $level = $murl['domain_level'] - 1;
            $host2 = $murl['domain_list'][$level - 1] . '.'
                   . $murl['domain_list'][$level];
            $host3 = $murl['domain_list'][$level - 2] . '.' . $host2;
            
            $buf = self::_gurl($server, $port, $host3);
            if (false === $buf) return self::_error(5);
            
            if ($buf) $response = self::_gresponse($buf) ? $buf : false;
            else $response = false;
            
            if ($response === false) {
                $buf = self::_gurl($server, $port, $host2);
                if (false === $buf) return self::_error(5);
                
                if ($buf) $response = self::_gresponse($buf) ? $buf : false;
                else $response = false;
            }
        }
        
        return $response === false ? self::_error(6) : $response;
    }
    
    /**
    * Возвращает описание последней ошибки либо false
    *
    * @return string|bool строка с расшифровкой ошибки либо false
    */
    static function get_error() {
        switch (self::$errnum) {
        case 1: $err = 'Не задан хост'; break;
        case 2: $err = 'Неверный формат url'; break;
        case 3: $err = 'Ошибка подключения к корневому серверу (whois.iana.org)'; break;
        case 4: $err = 'Не найден сервер для получения запрашиваемых данных'; break;
        case 5: $err = 'Ошибка подключения к серверу доменных имён (например, whois.ripn.net для доменов ru)'; break;
        case 6: $err = 'Данные по запрашиваемому домену не найдены'; break;
        }
        return isset($err) ? $err : false;
    }
    
    /**
    * Выполняет запрос к заданному хосту и возвращает ответ сервера
    *
    * @param string $host сервер для запроса
    * @param int $port порт на сервере
    * @param string $data данные, передаваеммые указанному серверу
    * @return string полученные данные
    *
    * @private
    */
    static private function _gurl($host, $port, $data) {
        $fp = @fsockopen($host, $port, $errno, $errstr, 3);
        if (false === $fp) return false;
        
        $buf = '';
        fputs($fp, $data . "\r\n");
        while ( ! feof($fp)) $buf .= fread($fp, 1024);
        fclose($fp);
        return $buf;
    }
    
    /**
    * Проверяет возвращённый запрос на корректность
    *
    * Whois-серверы всё-равно возвращают какой-либо ответ, даже если домен не
    * найден. Поэтому, данные желательно «профильтровать» на наличие в них
    * фраз, свидетельствующих о том, что домен не найден.
    *
    * @param string $buf информация по домену, полученная от whois-сервера
    * @return bool результат проверки; true если всё в порядке
    *
    * @private
    */
    static private function _gresponse($buf) {
        $stop_phrase = array(
            'No entries found',
            'No match for',
        );
        return ! preg_match('!('.implode('|', $stop_phrase).')!i', $buf);
    }
    
    /**
    * Устанавлиет номер ошибки и возвращеат false
    *
    * @param int $num номер ошибки
    * @result bool всегда false
    *
    * @private
    */
    static private function _error($num) {
        self::$errnum = $num;
        return false;
    }
}
